@@ -94,7 +94,9 @@ class Form(SignalEvent, TabContent):
                     attributes.get('mode') or ['tree', 'form']))
             and self.screen.local_context == attributes.get('context')
             and self.attributes.get('search_value') == (
-                attributes.get('search_value')))
+                attributes.get('search_value'))
+            and self.attributes.get('tab_domain') == (
+                attributes.get('tab_domain')))
 
     def __hash__(self):
         return id(self)
@@ -482,7 +484,9 @@ class Form(SignalEvent, TabContent):
         button_switch.props.sensitive = self.screen.number_of_views > 1
 
         msg = name + ' / ' + str(signal_data[1])
-        if signal_data[1] < signal_data[2]:
+        if (signal_data[1] < signal_data[2]
+                and self.screen.limit is not None
+                and signal_data[2] > self.screen.limit):
             msg += _(' of ') + str(signal_data[2])
         self.status_label.set_text(msg)
         self.message_info()
@@ -510,17 +514,18 @@ class Form(SignalEvent, TabContent):
             if value == 'ko':
                 record_id = self.screen.current_record.id
                 if self.sig_reload(test_modified=False):
-                    if self.screen.current_record:
+                    if record_id < 0:
+                        return None
+                    elif self.screen.current_record:
                         return record_id == self.screen.current_record.id
-                    elif record_id < 0:
-                        return True
             return False
         return True
 
     def sig_close(self, widget=None):
         for dialog in reversed(self.dialogs[:]):
             dialog.destroy()
-        return self.modified_save()
+        modified_save = self.modified_save()
+        return True if modified_save is None else modified_save
 
     def _action(self, action, atype):
         action = action.copy()
