@@ -38,7 +38,7 @@ class Record(SignalEvent):
         self.destroyed = False
 
     def __getitem__(self, name):
-        if name not in self._loaded and self.id >= 0:
+        if not self.destroyed and self.id >= 0 and name not in self._loaded:
             id2record = {
                 self.id: self,
                 }
@@ -80,7 +80,9 @@ class Record(SignalEvent):
                 limit = int(CONFIG['client.limit'] / len(fnames))
 
                 def filter_group(record):
-                    return name not in record._loaded and record.id >= 0
+                    return (not record.destroyed
+                        and record.id >= 0
+                        and name not in record._loaded)
 
                 def filter_parent_group(record):
                     return (filter_group(record)
@@ -303,7 +305,7 @@ class Record(SignalEvent):
     def pre_validate(self):
         if not self.modified_fields:
             return True
-        values = self._get_on_change_args(self.modified_fields)
+        values = self._get_on_change_args(['id'] + list(self.modified_fields))
         try:
             RPCExecute('model', self.model_name, 'pre_validate', values,
                 context=self.get_context())
@@ -399,8 +401,11 @@ class Record(SignalEvent):
 
     invalid_fields = property(_get_invalid_fields)
 
-    def get_context(self):
-        return self.group.context
+    def get_context(self, local=False):
+        if not local:
+            return self.group.context
+        else:
+            return self.group.local_context
 
     def set_default(self, val, signal=True, validate=True):
         fieldnames = []
