@@ -116,7 +116,7 @@ class CellCache(list):
 
 class Cell(object):
 
-    def set_editable(self, record):
+    def set_editable(self):
         pass
 
 
@@ -176,6 +176,7 @@ class Affix(Cell):
 class GenericText(Cell):
     align = 0
     editable = None
+    editing = None
 
     def __init__(self, view, attrs, renderer=None):
         super(GenericText, self).__init__()
@@ -264,15 +265,21 @@ class GenericText(Cell):
         if callback:
             callback()
 
-    def set_editable(self, record):
-        if not record or not self.editable:
+    def set_editable(self):
+        if not self.editable or not self.editing:
             return
+        record, field = self.editing
         self.editable.set_text(self.get_textual_value(record))
 
     def editing_started(self, cell, editable, path):
         def remove(editable):
             self.editable = None
+            self.editing = None
         self.editable = editable
+        store = self.view.treeview.get_model()
+        record = store.get_value(store.get_iter(path), 0)
+        field = record[self.attrs['name']]
+        self.editing = record, field
         editable.connect('remove-widget', remove)
         return False
 
@@ -407,6 +414,12 @@ class Time(Date):
         else:
             return ''
 
+    def set_editable(self):
+        if not self.editable or not self.editing:
+            return
+        record, field = self.editing
+        self.editable.get_child().set_text(self.get_textual_value(record))
+
 
 class TimeDelta(GenericText):
     align = 1
@@ -840,10 +853,10 @@ class Selection(GenericText, SelectionMixin, PopdownMixin):
         if callback:
             callback()
 
-    def set_editable(self, record):
-        if not record or not self.editable:
+    def set_editable(self):
+        if not self.editable and not self.editing:
             return
-        field = record[self.attrs['name']]
+        record, field = self.editing
         value = field.get(record)
         self.update_selection(record, field)
         self.set_popdown_value(self.editable, value)
@@ -901,7 +914,7 @@ class Reference(GenericText, SelectionMixin):
             callback()
 
 
-class ProgressBar(object):
+class ProgressBar(Cell):
     align = 0.5
     orientations = {
         'left_to_right': gtk.PROGRESS_LEFT_TO_RIGHT,
