@@ -1,23 +1,27 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-from tryton.common import TRYTON_ICON, COLOR_SCHEMES
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from tryton.common import TRYTON_ICON
 import tryton.common as common
-from tryton.config import CONFIG
 import gtk
 import pango
 import gettext
+
+from .infobar import InfoBar
+from tryton.common.domain_parser import quote
+from tryton.common.underline import set_underline
+from tryton.gui import Main
 from tryton.gui.window.nomodal import NoModal
-from tryton.common import RPCExecute, RPCException
 
 _ = gettext.gettext
 
 
-class WinForm(NoModal):
+class WinForm(NoModal, InfoBar):
     "Form window"
 
     def __init__(self, screen, callback, view_type='form',
             new=False, many=0, domain=None, context=None,
-            save_current=False):
+            save_current=False, title='', rec_name=None):
+        tooltips = common.Tooltips()
         NoModal.__init__(self)
         self.screen = screen
         self.callback = callback
@@ -25,23 +29,32 @@ class WinForm(NoModal):
         self.domain = domain
         self.context = context
         self.save_current = save_current
+        self.title = title
         self.prev_view = self.screen.current_view
         self.screen.screen_container.alternate_view = True
-        if view_type not in (x.view_type for x in self.screen.views) and \
-                view_type not in self.screen.view_to_load:
-            self.screen.add_view_id(None, view_type)
         self.screen.switch_view(view_type=view_type)
+        if self.screen.current_view.view_type != view_type:
+            self.destroy()
+            return
         if new:
-            self.screen.new()
+            self.screen.new(rec_name=rec_name)
         self.win = gtk.Dialog(_('Link'), self.parent,
                 gtk.DIALOG_DESTROY_WITH_PARENT)
+        Main().add_window(self.win)
         self.win.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.win.set_icon(TRYTON_ICON)
-        self.win.set_has_separator(False)
         self.win.set_deletable(False)
+        self.win.connect('delete-event', lambda *a: True)
         self.win.connect('close', self.close)
         self.win.connect('response', self.response)
 
+        allocation = self.parent.get_allocation()
+        width, height, = allocation.width, allocation.height
+        if self.parent != self.sensible_widget:
+            width = max(width - 150, 0)
+            height = max(height - 150, 0)
+        self.win.set_default_size(width, height)
+
         self.accel_group = gtk.AccelGroup()
         self.win.add_accel_group(self.accel_group)
 
@@ -50,62 +63,69 @@ class WinForm(NoModal):
         self.but_ok = None
         self.but_new = None
 
+        self._initial_value = None
         if view_type == 'form':
-            if not new and self.screen.current_record.id < 0:
-                stock_id = gtk.STOCK_DELETE
+            if new:
+                label, icon = _("Delete"), 'tryton-delete'
             else:
-                stock_id = gtk.STOCK_CANCEL
-            self.but_cancel = self.win.add_button(stock_id,
-                gtk.RESPONSE_CANCEL)
+                label, icon = _("Cancel"), 'tryton-cancel'
+                record = self.screen.current_record
+                self._initial_value = record.get_on_change_value()
+                if record.parent and record.parent_name in record.group.fields:
+                    parent_field = record.group.fields[record.parent_name]
+                    self._initial_value[record.parent_name] = (
+                        parent_field.get_eval(record))
+            self.but_cancel = self.win.add_button(
+                set_underline(label), gtk.RESPONSE_CANCEL)
+            self.but_cancel.set_image(common.IconFactory.get_image(
+                    icon, gtk.ICON_SIZE_BUTTON))
+            self.but_cancel.set_always_show_image(True)
 
         if new and self.many:
-            self.but_new = self.win.add_button(gtk.STOCK_NEW,
-                gtk.RESPONSE_ACCEPT)
+            self.but_new = self.win.add_button(
+                set_underline(_("New")), gtk.RESPONSE_ACCEPT)
+            self.but_new.set_image(common.IconFactory.get_image(
+                    'tryton-create', gtk.ICON_SIZE_BUTTON))
+            self.but_new.set_always_show_image(True)
             self.but_new.set_accel_path('<tryton>/Form/New', self.accel_group)
 
         if self.save_current:
-            self.but_ok = gtk.Button(_('_Save'))
-            img_save = gtk.Image()
-            img_save.set_from_stock('tryton-save', gtk.ICON_SIZE_BUTTON)
-            self.but_ok.set_image(img_save)
+            self.but_ok = gtk.Button(_('_Save'), use_underline=True)
+            self.but_ok.set_image(common.IconFactory.get_image(
+                    'tryton-save', gtk.ICON_SIZE_BUTTON))
+            self.but_ok.set_always_show_image(True)
             self.but_ok.set_accel_path('<tryton>/Form/Save', self.accel_group)
             self.but_ok.set_can_default(True)
             self.but_ok.show()
             self.win.add_action_widget(self.but_ok, gtk.RESPONSE_OK)
+            if not new:
+                self.but_ok.props.sensitive = False
         else:
-            self.but_ok = self.win.add_button(gtk.STOCK_OK,
-                gtk.RESPONSE_OK)
+            self.but_ok = self.win.add_button(
+                set_underline(_("OK")), gtk.RESPONSE_OK)
+            self.but_ok.set_image(common.IconFactory.get_image(
+                    'tryton-ok', gtk.ICON_SIZE_BUTTON))
+            self.but_ok.set_always_show_image(True)
         self.but_ok.add_accelerator('clicked', self.accel_group,
             gtk.keysyms.Return, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
         self.win.set_default_response(gtk.RESPONSE_OK)
 
-        self.win.set_title(self.screen.current_view.title)
+        self.win.set_title(self.title)
 
         title = gtk.Label()
         title.modify_font(pango.FontDescription("bold 12"))
-        title.set_label(self.screen.current_view.title)
+        title.set_label(common.ellipsize(self.title, 80))
+        tooltips.set_tip(title, self.title)
         title.set_padding(20, 3)
         title.set_alignment(0.0, 0.5)
         title.set_size_request(0, -1)  # Allow overflow
+        title.set_max_width_chars(1)
+        title.set_ellipsize(pango.ELLIPSIZE_END)
         title.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))
         title.show()
 
-        self.info_label = gtk.Label()
-        self.info_label.set_padding(3, 3)
-        self.info_label.set_alignment(1.0, 0.5)
-
-        self.eb_info = gtk.EventBox()
-        self.eb_info.add(self.info_label)
-        self.eb_info.connect('button-release-event',
-                lambda *a: self.message_info(''))
-
-        vbox = gtk.VBox()
-        vbox.pack_start(self.eb_info, expand=True, fill=True, padding=5)
-        vbox.show()
-
         hbox = gtk.HBox()
         hbox.pack_start(title, expand=True, fill=True)
-        hbox.pack_start(vbox, expand=False, fill=True, padding=20)
         hbox.show()
 
         frame = gtk.Frame()
@@ -122,9 +142,37 @@ class WinForm(NoModal):
 
         if view_type == 'tree':
             hbox = gtk.HBox(homogeneous=False, spacing=0)
-            tooltips = common.Tooltips()
             access = common.MODELACCESS[screen.model_name]
 
+            but_switch = gtk.Button()
+            tooltips.set_tip(but_switch, _('Switch'))
+            but_switch.connect('clicked', self.switch_view)
+            but_switch.add(common.IconFactory.get_image(
+                    'tryton-switch', gtk.ICON_SIZE_SMALL_TOOLBAR))
+            but_switch.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(but_switch, expand=False, fill=False)
+
+            self.but_pre = gtk.Button()
+            tooltips.set_tip(self.but_pre, _('Previous'))
+            self.but_pre.connect('clicked', self._sig_previous)
+            self.but_pre.add(common.IconFactory.get_image(
+                    'tryton-back', gtk.ICON_SIZE_SMALL_TOOLBAR))
+            self.but_pre.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(self.but_pre, expand=False, fill=False)
+
+            self.label = gtk.Label('(0,0)')
+            hbox.pack_start(self.label, expand=False, fill=False)
+
+            self.but_next = gtk.Button()
+            tooltips.set_tip(self.but_next, _('Next'))
+            self.but_next.connect('clicked', self._sig_next)
+            self.but_next.add(common.IconFactory.get_image(
+                    'tryton-forward', gtk.ICON_SIZE_SMALL_TOOLBAR))
+            self.but_next.set_relief(gtk.RELIEF_NONE)
+            hbox.pack_start(self.but_next, expand=False, fill=False)
+
+            hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+
             if domain is not None:
                 self.wid_text = gtk.Entry()
                 self.wid_text.set_property('width_chars', 13)
@@ -135,11 +183,8 @@ class WinForm(NoModal):
                 self.but_add = gtk.Button()
                 tooltips.set_tip(self.but_add, _('Add'))
                 self.but_add.connect('clicked', self._sig_add)
-                img_add = gtk.Image()
-                img_add.set_from_stock('tryton-list-add',
-                    gtk.ICON_SIZE_SMALL_TOOLBAR)
-                img_add.set_alignment(0.5, 0.5)
-                self.but_add.add(img_add)
+                self.but_add.add(common.IconFactory.get_image(
+                        'tryton-add', gtk.ICON_SIZE_SMALL_TOOLBAR))
                 self.but_add.set_relief(gtk.RELIEF_NONE)
                 hbox.pack_start(self.but_add, expand=False, fill=False)
                 if not access['read'] or readonly:
@@ -148,11 +193,8 @@ class WinForm(NoModal):
                 self.but_remove = gtk.Button()
                 tooltips.set_tip(self.but_remove, _('Remove <Del>'))
                 self.but_remove.connect('clicked', self._sig_remove, True)
-                img_remove = gtk.Image()
-                img_remove.set_from_stock('tryton-list-remove',
-                    gtk.ICON_SIZE_SMALL_TOOLBAR)
-                img_remove.set_alignment(0.5, 0.5)
-                self.but_remove.add(img_remove)
+                self.but_remove.add(common.IconFactory.get_image(
+                        'tryton-remove', gtk.ICON_SIZE_SMALL_TOOLBAR))
                 self.but_remove.set_relief(gtk.RELIEF_NONE)
                 hbox.pack_start(self.but_remove, expand=False, fill=False)
                 if not access['read'] or readonly:
@@ -163,10 +205,8 @@ class WinForm(NoModal):
             self.but_new = gtk.Button()
             tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
             self.but_new.connect('clicked', self._sig_new)
-            img_new = gtk.Image()
-            img_new.set_from_stock('tryton-new', gtk.ICON_SIZE_SMALL_TOOLBAR)
-            img_new.set_alignment(0.5, 0.5)
-            self.but_new.add(img_new)
+            self.but_new.add(common.IconFactory.get_image(
+                    'tryton-create', gtk.ICON_SIZE_SMALL_TOOLBAR))
             self.but_new.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_new, expand=False, fill=False)
             if not access['create'] or readonly:
@@ -175,11 +215,8 @@ class WinForm(NoModal):
             self.but_del = gtk.Button()
             tooltips.set_tip(self.but_del, _('Delete selected record <Del>'))
             self.but_del.connect('clicked', self._sig_remove, False)
-            img_del = gtk.Image()
-            img_del.set_from_stock('tryton-delete',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-            img_del.set_alignment(0.5, 0.5)
-            self.but_del.add(img_del)
+            self.but_del.add(common.IconFactory.get_image(
+                    'tryton-delete', gtk.ICON_SIZE_SMALL_TOOLBAR))
             self.but_del.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_del, expand=False, fill=False)
             if not access['delete'] or readonly:
@@ -189,56 +226,13 @@ class WinForm(NoModal):
             tooltips.set_tip(self.but_undel,
                 _('Undelete selected record <Ins>'))
             self.but_undel.connect('clicked', self._sig_undelete)
-            img_undel = gtk.Image()
-            img_undel.set_from_stock('tryton-undo',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-            img_undel.set_alignment(0.5, 0.5)
-            self.but_undel.add(img_undel)
+            self.but_undel.add(common.IconFactory.get_image(
+                    'tryton-undo', gtk.ICON_SIZE_SMALL_TOOLBAR))
             self.but_undel.set_relief(gtk.RELIEF_NONE)
             hbox.pack_start(self.but_undel, expand=False, fill=False)
             if not access['delete'] or readonly:
                 self.but_undel.set_sensitive(False)
 
-            hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
-
-            self.but_pre = gtk.Button()
-            tooltips.set_tip(self.but_pre, _('Previous'))
-            self.but_pre.connect('clicked', self._sig_previous)
-            img_pre = gtk.Image()
-            img_pre.set_from_stock('tryton-go-previous',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-            img_pre.set_alignment(0.5, 0.5)
-            self.but_pre.add(img_pre)
-            self.but_pre.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(self.but_pre, expand=False, fill=False)
-
-            self.label = gtk.Label('(0,0)')
-            hbox.pack_start(self.label, expand=False, fill=False)
-
-            self.but_next = gtk.Button()
-            tooltips.set_tip(self.but_next, _('Next'))
-            self.but_next.connect('clicked', self._sig_next)
-            img_next = gtk.Image()
-            img_next.set_from_stock('tryton-go-next',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-            img_next.set_alignment(0.5, 0.5)
-            self.but_next.add(img_next)
-            self.but_next.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(self.but_next, expand=False, fill=False)
-
-            hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
-
-            but_switch = gtk.Button()
-            tooltips.set_tip(but_switch, _('Switch'))
-            but_switch.connect('clicked', self.switch_view)
-            img_switch = gtk.Image()
-            img_switch.set_from_stock('tryton-fullscreen',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-            img_switch.set_alignment(0.5, 0.5)
-            but_switch.add(img_switch)
-            but_switch.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(but_switch, expand=False, fill=False)
-
             but_switch.props.sensitive = screen.number_of_views > 1
 
             tooltips.enable()
@@ -258,9 +252,8 @@ class WinForm(NoModal):
 
         scroll.add(self.screen.screen_container.alternate_viewport)
 
-        sensible_allocation = self.sensible_widget.get_allocation()
-        self.win.set_default_size(int(sensible_allocation.width * 0.9),
-            int(sensible_allocation.height * 0.9))
+        self.create_info_bar()
+        self.win.vbox.pack_start(self.info_bar, False, True)
 
         if view_type == 'tree':
             self.screen.signal_connect(self, 'record-message', self._sig_label)
@@ -274,23 +267,11 @@ class WinForm(NoModal):
                 self.activate_save)
 
         self.register()
-        self.win.show()
-
-        common.center_window(self.win, self.parent, self.sensible_widget)
+        self.show()
 
         self.screen.display()
         self.screen.current_view.set_cursor()
 
-    def message_info(self, message, color='red'):
-        if message:
-            self.info_label.set_label(message)
-            self.eb_info.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(
-                COLOR_SCHEMES.get(color, 'white')))
-            self.eb_info.show_all()
-        else:
-            self.info_label.set_label('')
-            self.eb_info.hide()
-
     def on_keypress(self, widget, event):
         if (event.keyval == gtk.keysyms.F3) \
                 and self.but_new.get_property('sensitive'):
@@ -332,17 +313,7 @@ class WinForm(NoModal):
         from tryton.gui.window.win_search import WinSearch
         domain = self.domain[:]
         model_name = self.screen.model_name
-
-        try:
-            if self.wid_text.get_text():
-                dom = [('rec_name', 'ilike',
-                        '%' + self.wid_text.get_text() + '%'), domain]
-            else:
-                dom = domain
-            ids = RPCExecute('model', model_name, 'search', dom,
-                    0, CONFIG['client.limit'], None, context=self.context)
-        except RPCException:
-            return False
+        value = self.wid_text.get_text()
 
         def callback(result):
             if result:
@@ -351,11 +322,11 @@ class WinForm(NoModal):
                 self.screen.display(res_id=ids[0])
             self.screen.set_cursor()
             self.wid_text.set_text('')
-        if len(ids) != 1:
-            WinSearch(model_name, callback, sel_multi=True,
-                ids=ids, context=self.context, domain=domain)
-        else:
-            callback([(i, None) for i in ids])
+
+        win = WinSearch(model_name, callback, sel_multi=True,
+            context=self.context, domain=domain)
+        win.screen.search_filter(quote(value))
+        win.show()
 
     def _sig_label(self, screen, signal_data):
         name = '_'
@@ -388,9 +359,11 @@ class WinForm(NoModal):
 
     def activate_save(self, *args):
         modified = self.screen.modified()
-        self.but_ok.props.sensitive = modified
+        # Keep sensible as change could have been trigger by a Many2One edition
+        sensitive = modified or self.but_ok.props.sensitive
+        self.but_ok.props.sensitive = sensitive
         self.win.set_default_response(
-            gtk.RESPONSE_OK if modified else gtk.RESPONSE_CANCEL)
+            gtk.RESPONSE_OK if sensitive else gtk.RESPONSE_CANCEL)
 
     def close(self, widget):
         widget.emit_stop_by_name('close')
@@ -414,7 +387,7 @@ class WinForm(NoModal):
                     validate = False
             elif validate and self.screen.current_view.view_type == 'form':
                 view = self.screen.current_view
-                for widgets in view.widgets.itervalues():
+                for widgets in view.widgets.values():
                     for widget in widgets:
                         if (hasattr(widget, 'screen')
                                 and widget.screen.pre_validate):
@@ -422,23 +395,29 @@ class WinForm(NoModal):
                             if record:
                                 validate = record.pre_validate()
             if not validate:
+                self.message_info(self.screen.invalid_message(),
+                    gtk.MESSAGE_ERROR)
                 self.screen.set_cursor()
                 self.screen.display()
                 return
+            self.message_info()
             if response_id == gtk.RESPONSE_ACCEPT:
                 self.new()
                 return
         if (self.screen.current_record
                 and not readonly
                 and response_id in cancel_responses):
+            record = self.screen.current_record
+            added = 'id' in record.modified_fields
             if (self.screen.current_record.id < 0
                     or self.save_current):
-                self.screen.group.remove(self.screen.current_record,
-                    remove=True)
-            elif self.screen.current_record.modified:
-                self.screen.current_record.cancel()
-                self.screen.current_record.reload()
-                self.screen.current_record.signal('record-changed')
+                self.screen.cancel_current(self._initial_value)
+            elif record.modified:
+                record.cancel()
+                record.reload()
+                record.signal('record-changed')
+            if added:
+                record.modified_fields.setdefault('id')
             result = False
         else:
             result = response_id not in cancel_responses
@@ -447,6 +426,7 @@ class WinForm(NoModal):
 
     def new(self):
         self.screen.new()
+        self._initial_value = None
         self.screen.current_view.display()
         self.screen.set_cursor(new=True)
         self.many -= 1
@@ -457,16 +437,16 @@ class WinForm(NoModal):
     def destroy(self):
         self.screen.screen_container.alternate_view = False
         viewport = self.screen.screen_container.alternate_viewport
-        if viewport.get_child():
-            viewport.remove(viewport.get_child())
+        if viewport and viewport.get_parent():
+            viewport.get_parent().remove(viewport)
         self.screen.switch_view(view_type=self.prev_view.view_type)
         self.screen.signal_unconnect(self)
-        self.win.destroy()
+        if getattr(self, 'win', None):
+            self.win.destroy()
         NoModal.destroy(self)
 
     def show(self):
         self.win.show()
-        common.center_window(self.win, self.parent, self.sensible_widget)
 
     def hide(self):
         self.win.hide()
